pub struct Approx {
pub value: BigInt,
pub precision: i32,
}Expand description
An approximation of a real number to a certain precision.
If the underlying value is x, then it is guaranteed that
|x - value*2^precision| < 2^precision.
Fields
value: BigIntprecision: i32Trait Implementations